@@ -425,7 +425,6 @@ static void
 giggle_git_update_remotes (GiggleGit* git)
 {
 	GiggleGitPriv *priv;
-	GError        *error;
 	gchar         *path;
 	GDir          *dir;
 
@@ -437,13 +436,10 @@ giggle_git_update_remotes (GiggleGit* git)
 	priv->remotes = NULL;
 
 	/* list files and add them */
-	error = NULL;
 	path = g_build_filename (priv->git_dir, "remotes", NULL);
-	dir = g_dir_open (path, 0, &error);
+	dir = g_dir_open (path, 0, NULL);
 
-	if(error) {
-		g_warning ("Error loading remotes: %s", error->message);
-	} else {
+	if (dir) {
 		const gchar *file;
 
 		for(file = g_dir_read_name(dir); file; file = g_dir_read_name(dir)) {
@@ -523,6 +519,7 @@ giggle_git_set_directory (GiggleGit    *git,
 	GiggleGitPriv *priv;
 	gchar         *tmp_dir;
 	gchar         *suffix;
+	gchar         *dir;
 
 	g_return_val_if_fail (GIGGLE_IS_GIT (git), FALSE);
 	g_return_val_if_fail (directory != NULL, FALSE);
@@ -534,8 +531,9 @@ giggle_git_set_directory (GiggleGit    *git,
 	}
 
 	/* update working directory */
+	dir = g_strdup (directory);
 	g_free (priv->directory);
-	priv->directory = g_strdup (directory);
+	priv->directory = dir;
 
 	/* update git dir */
 	g_free (priv->git_dir);
